/*
 * custom.css
 * Copyright (C) 2017 Adrian Perez
 *
 * Distributed under terms of the GPLv3 license.
 */

.titlebar.revolt-slim {
	min-height: 0;
}

.titlebar.revolt-slim separator,
.titlebar.revolt-slim button {
	margin: 3px 0 3px 0;
}

.titlebar.revolt-slim button {
	min-height: 16px;
	min-width: 18px;
	padding: 3px 4px 3px 4px;
}
